﻿using UnityEngine;
using System.IO;
using System;
using HarmonyLib;

public class ScatteredRemains : Mod
{
    Harmony harmony;
    static public JsonModInfo modInfo;
    static string configPath = Path.Combine(SaveAndLoad.WorldPath, "ScatteredRemains.json");
    public static JSONObject Config = getSaveJson();
    public static System.Random RNG = new System.Random();
    public static string[] YesNos = { "true", "false", "yes", "no", "1", "0" };
    public static float dropChance
    {
        get
        {
            if (Config.IsNull || !Config.HasField("chance"))
                return 1f;
            return Config.GetField("chance").n;
        }
        set
        {
            if (!Config.IsNull && Config.HasField("chance"))
                Config.SetField("chance", value);
            else
                Config.AddField("chance", value);
        }
    }
    public static bool alwaysSpecial
    {
        get
        {
            if (Config.IsNull || !Config.HasField("specialsAlwaysDrop"))
                return false;
            return Config.GetField("specialsAlwaysDrop").b;
        }
        set
        {
            if (!Config.IsNull && Config.HasField("specialsAlwaysDrop"))
                Config.SetField("specialsAlwaysDrop", value);
            else
                Config.AddField("specialsAlwaysDrop", value);
        }
    }

    public void Start()
    {
        modInfo = modlistEntry.jsonmodinfo;
        harmony = new Harmony("com.aidanamite.ScatteredRemains");
        harmony.PatchAll();
        Log("Mod has been loaded!");
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll();
        Debug.Log("Mod has been unloaded!");
    }

    public static void Log(object message)
    {
        Debug.Log("[" + modInfo.name + "]: " + message.ToString());
    }

    public static void ErrorLog(object message)
    {
        Debug.LogError("[" + modInfo.name + "]: " + message.ToString());
    }

    private static JSONObject getSaveJson()
    {
        JSONObject data;
        try
        {
            data = new JSONObject(File.ReadAllText(configPath));
        }
        catch
        {
            data = JSONObject.Create();
            saveJson(data);
        }
        return data;
    }

    private static void saveJson(JSONObject data)
    {
        try
        {
            File.WriteAllText(configPath, data.ToString());
        }
        catch (Exception err)
        {
            ErrorLog("An error occured while trying to save settings: " + err.Message);
        }
    }

    [ConsoleCommand(name: "setDeathDropChance", docs: "Syntax: 'setDeathDropChance <chance>'  Changes the chance of items dropping when you respawn. Use decimal values from 1 to 0. 1 is 100% chance")]
    public static string MyCommand(string[] args)
    {
        if (args.Length < 1)
            return "Not enough arguments";
        if (args.Length > 1)
            return "Too many arguments";
        try
        {
            dropChance = float.Parse(args[0]);
            saveJson(Config);
            return "Drop chance is now " + Math.Round(dropChance * 100) + "%";
        }
        catch
        {
            return "Cannot parse " + args[0] + " as a number";
        }
    }

    [ConsoleCommand(name: "setAlwaysDropSpecial", docs: "Syntax: 'setAlwaysDropSpecial <true|false>'  Sets weither or not \"special\" items such as equipment and placeables should be dropped regardless of drop chance")]
    public static string MyCommand2(string[] args)
    {
        if (args.Length < 1)
            return "Not enough arguments";
        if (args.Length > 1)
            return "Too many arguments";
        int responce = getYesNo(args[0]);
        if (responce == -1)
            return args[0] + " is not a valid input";
        alwaysSpecial = (new bool[] { true, false})[responce];
        saveJson(Config);
        return "Now special items will" + (alwaysSpecial ? "" : " not") + " always drop";
    }

    public static bool isItemSpecial(Network_Player player, ItemInstance item)
    {
        if (item.settings_buildable != null && item.settings_buildable.Placeable) return true;
        if (item.settings_equipment != null && item.settings_equipment.EquipType != EquipSlotType.None) return true;
        foreach (ItemConnection connect in player.PlayerItemManager.useItemController.allConnections)
            if (connect.inventoryItem == item.baseItem)
                return true;
        return false;
    }

    public static int getYesNo(string parameter)
    {
        parameter = parameter.ToLower();
        for (int i = 0;i < YesNos.Length;i++)
            if (YesNos[i].StartsWith(parameter))
                return i % 2;
        return -1;
    }

    public void ExtraSettingsAPI_SettingsOpen()
    {
        ExtraSettingsAPI_SetCheckboxState("Always Drop Special Items", alwaysSpecial);
        ExtraSettingsAPI_SetSliderValue("Drop Chance", dropChance);

    }

    public void ExtraSettingsAPI_SettingsClose()
    {
        alwaysSpecial = ExtraSettingsAPI_GetCheckboxState("Always Drop Special Items");
        dropChance = ExtraSettingsAPI_GetSliderRealValue("Drop Chance");
        saveJson(Config);
    }


    static Traverse ExtraSettingsAPI_Traverse;
    static bool ExtraSettingsAPI_Loaded = false;
    public bool ExtraSettingsAPI_GetCheckboxState(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getCheckboxState", new object[] { this, SettingName }).GetValue<bool>();
        return false;
    }
    public float ExtraSettingsAPI_GetSliderRealValue(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getSliderRealValue", new object[] { this, SettingName }).GetValue<float>();
        return 0;
    }
    public void ExtraSettingsAPI_SetCheckboxState(string SettingName, bool value)
    {
        if (ExtraSettingsAPI_Loaded)
            ExtraSettingsAPI_Traverse.Method("setCheckboxState", new object[] { this, SettingName, value }).GetValue<bool>();
    }
    public void ExtraSettingsAPI_SetSliderValue(string SettingName, float value)
    {
        if (ExtraSettingsAPI_Loaded)
            ExtraSettingsAPI_Traverse.Method("setSliderValue", new object[] { this, SettingName, value }).GetValue<float>();
    }
}

[HarmonyPatch(typeof(Player), "StartRespawn")]
public class Patch_PlayerRespawn
{
    static void Prefix(ref Player __instance, ref bool clearInventory)
    {
        Network_Player player = Traverse.Create(__instance).Field("playerNetwork").GetValue<Network_Player>();
        if (player.IsLocalPlayer && clearInventory)
            foreach (Slot slot in player.Inventory.allSlots)
                if (!slot.IsEmpty)
                {
                    if (!(ScatteredRemains.alwaysSpecial && ScatteredRemains.isItemSpecial(player, slot.itemInstance)))
                    {
                        int itemCount = slot.itemInstance.Amount;
                        int newCount = 0;
                        for (int i = 0; i < itemCount; i++)
                            if (ScatteredRemains.RNG.NextDouble() < ScatteredRemains.dropChance)
                                newCount++;
                        if (newCount < itemCount)
                        {
                            if (newCount == 0)
                                slot.SetItem(null);
                            else
                                slot.itemInstance.Amount = newCount;
                        }
                    }
                    if (!slot.IsEmpty)
                    {
                        Helper.DropItem(slot.itemInstance, player.transform.position, new Vector3((float)ScatteredRemains.RNG.NextDouble() - 0.5f, (float)ScatteredRemains.RNG.NextDouble() - 0.5f, (float)ScatteredRemains.RNG.NextDouble() - 0.5f), player.PersonController.HasRaftAsParent);
                    }
                    //player.Inventory.DropItem(slot);
                }
    }
}